/***************************************************************************
 *   Copyright (C) 2006 by George Kiagiadakis   *
 *   gkiagia@users.sourceforge.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "scriptmaker.h"
#include <QMessageBox>
#include <QFile>
#include <QFileDialog>
#include <QTextStream>
#include "macros.h"
#include "configdelegate.h"

scriptMaker::scriptMaker(QWidget *parent)
 : QDialog(parent)
{
	sm.setupUi(this);
	cfgHandler->connectListView(sm.cfgList);

	DummyDelegate *delegate = new DummyDelegate;
	sm.cfgList->setItemDelegate( delegate );
}

scriptMaker::~scriptMaker()
{
	cfgHandler->disconnectListView(sm.cfgList);
}

void scriptMaker::on_winappBrowse_clicked()
{
	QString winApp = QFileDialog::getOpenFileName( this, "Choose a windows program", cfgHandler->getCurrentCfgPath(sm.cfgList) , "WINE executables (*.exe *.exe.so)" );

	sm.winappEdit->setText(winApp);

	if (sm.workdirEdit->text().isEmpty() && !winApp.isEmpty())
		sm.workdirEdit->setText( QFileInfo(winApp).absolutePath() );

	if (sm.nameEdit->text().isEmpty() && !winApp.isEmpty())
		sm.nameEdit->setText( QFileInfo(winApp).baseName() );

	raise();
	activateWindow();
}

void scriptMaker::on_workdirBrowse_clicked()
{
	sm.workdirEdit->setText( QFileDialog::getExistingDirectory( this, "Select the working directory", cfgHandler->getCurrentCfgPath(sm.cfgList), QFileDialog::ShowDirsOnly) );

	raise();
	activateWindow();
}

void scriptMaker::on_okButton_clicked()
{
	if ( sm.nameEdit->text().isEmpty() || sm.winappEdit->text().isEmpty() )
	{
		QMessageBox::information(this, "Not enough info", "You must provide a program name and a script name", "OK");
		raise();
		activateWindow();
		return;
	}

	QFile s( QtWine::settings->value("scriptmaker/script_dir", QDir::homePath()+="/bin/").toString() += sm.nameEdit->text() );
	if ( !s.open(QIODevice::WriteOnly) )
	{
		QMessageBox::critical(this, "Error", "Could not open file", "OK");
		raise();
		activateWindow();
		return;
	}
	
	QTextStream script(&s);
	script << "#!/bin/sh\n";
	script << "#This script was automatically generated by QtWine Launch Script Maker\n";
	script << "#More info at qtwine.sourceforge.net\n" << endl;

	const char w = ' '; // white space
	const char Q = '\"'; // quotes

	script << "qtwine" << w << "--launch" << w << Q << cfgHandler->getCurrentCfgName(sm.cfgList) << Q << w;

	if ( sm.silentBox->isChecked() )
		script << "--silent" << w;

	if ( !sm.workdirEdit->text().isEmpty() )
		script << "--workdir" << w << Q << sm.workdirEdit->text() << Q << w;

	script << Q << sm.winappEdit->text() << Q << w << sm.argsEdit->text() << endl;

	s.close();

	// make executable. Set permissions to rwxr__r__
	s.setPermissions( QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner | QFile::ReadGroup | QFile::ReadOther );

	QMessageBox::information(this, "Success", "Script created successfully!", "OK"); // FIXME Are you sure?
	done(QDialog::Accepted); // close scriptmaker
}
